import unittest
import jsonofabitch as jsob
import json

datajsob = [
    '"bob":g=5, h=4 ; b=(6,6,3,), f=4, l=[3,3,3,3,]',
    '"bob":g=5, h=4 ;; b=(6,6,3,), f=4, l=[3.0,3.0,3.0,3,]',
    '"bob":g=5, h=4 ;, b=(6,6,3,), f=4, l=[3,"3",e3,3,]',
    '"bob":g=5, h=4 ;, b=(6,6,3,), f=4, l=[3,"3",e:3;;3,]',
]


datacheck = [
    {"bob": {"g": 5, "h": 4, "b": (6, 6, 3), "f": 4, "l": [3, 3, 3, 3]}},
    {"bob": {"g": 5, "h": 4}, "b": (6, 6, 3), "f": 4, "l": [3.0, 3.0, 3.0, 3]},
    {"bob": {"g": 5, "h": 4}, "b": (6, 6, 3), "f": 4, "l": [3, "3", "e3", 3]},
    {"bob": {"g": 5, "h": 4}, "b": (6, 6, 3), "f": 4, "l": [3, "3", {"e": 3}, 3]},
]


class TestJSOB(unittest.TestCase):
    def test_loads(self):
        for j, d in enumerate(datajsob):
            self.assertEqual(jsob.loads(d), datacheck[j])

    def test_num(self):
        data = [
            "d=3.0",
            "d=3",
            "d=3e3",
            "d=3.0e3",
            "d=e3",
            "d=(2,2)",
            "d=[3,4,5]",
            "d=g:3,h:3",
            "d=true",
            "d=false",
        ]
        dout = [float, int, float, float, str, tuple, list, dict, bool, bool]
        for j, d in enumerate(data):
            self.assertTrue(type(jsob.loads(d)["d"]) is dout[j])

    def test_empty_plus_lists(self):
        data = [
            "{}",
            "",
            "sdfsdf",
            "5",
            "[5,5]",
            "[d=5,h=3]",
            "[d=5;,h=3]",
            "[d=5,,h=3,]",
            "[d=5;;h=3]",
            "(d=5;;h=3)",
            "(d=5;h=3;)",
            "{d=5;h=3;}",
            "{d=5,h=3,}",
            "d=5;h=3",
            "d=5,h=3,",
            "d=5,h=3;",
            "{d:d=5,h=3,}",
            "{d:d=5,,h=3,}",
            "d=d:5;;h=3",
        ]
        dout = [
            {},
            {},
            "sdfsdf",
            5,
            [5, 5],
            [{"d": 5, "h": 3}],
            [{"d": 5}, {"h": 3}],
            [{"d": 5}, {"h": 3}],
            [{"d": 5}, {"h": 3}],
            ({"d": 5}, {"h": 3}),
            ({"d": 5, "h": 3},),
            {"d": 5, "h": 3},
            {"d": 5, "h": 3},
            {"d": 5, "h": 3},
            {"d": 5, "h": 3},
            {"d": 5, "h": 3},
            {"d": {"d": 5, "h": 3}},
            {"d": {"d": 5}, "h": 3},
            {"d": {"d": 5}, "h": 3},
        ]
        for j, d in enumerate(data):
            self.assertEqual(jsob.loads(d), dout[j])

    def test_special(self):
        data = [
            "d=3.0e3",
            "d=true",
            "d=false",
            "d=null",
        ]
        dout = [3000.0, True, False, None]
        for j, d in enumerate(data):
            self.assertEqual(jsob.loads(d)["d"], dout[j])

    def test_dumps_compliance(self):
        for d in datajsob:
            di = jsob.loads(d)
            self.assertEqual(json.loads(json.dumps(di)), jsob.loads(json.dumps(di)))
            self.assertEqual(json.loads(jsob.dumps(di)), json.loads(json.dumps(di)))

    def test_dumps_tuples_self_compliant(self):
        for d in datajsob:
            di = jsob.loads(d)
            self.assertEqual(jsob.loads(jsob.dumps(di, tuples=True)), di)

    def test_dumpslob(self):
        for j in range(500):
            for d in datajsob:
                di = jsob.loads(d)
                du = jsob.dumpslob(di)
                try:
                    self.assertEqual(jsob.loads(du), di)
                except Exception as e:
                    print("SONOFADUMPS FAILING STRING: ", du)
                    raise (e)

if __name__ == "__main__":
    unittest.main()



import sys

__all__ = (
    "loads",
    "dumps",
    "correct",
    "dumpslob",
)

from .jsob import Lark_StandAlone, Transformer, v_args
from random import random

json_grammar = r"""
    ?start: value

    ?value: object
          | array
          | tuple
          | string
          | SIGNED_NUMBER      -> number
          | "true"             -> true
          | "false"            -> false
          | "null"             -> null

    array  : "[" [value ( (","|";" )value )*] [","|";"]"]"
    tuple  : "(" [value ( (","|";" ) value )*] [","|";"]")"
    object : ["{"] pair [ ((","|";" ) pair )*] [","|";"] ["}"]
    pair   : string (":" | "=") value 
    
    string : CNAME|ESCAPED_STRING 

    %import common.CNAME
    %import common.ESCAPED_STRING
    %import common.SIGNED_NUMBER
    %import common.WS

    %ignore WS
"""


class TreeToJson(Transformer):
    @v_args(inline=True)
    def string(self, s):
        return s.strip('"').replace('\\"', '"')

    @v_args(inline=True)
    def numb(self, s):
        if "." in s or "e" in s:
            return float(s)
        return int(s)

    tuple = tuple
    array = list
    pair = tuple
    object = dict
    number = v_args(inline=True)(numb)

    null = lambda self, _: None
    true = lambda self, _: True
    false = lambda self, _: False


json_parser = Lark_StandAlone(transformer=TreeToJson())
# json_parser = Lark(
#    json_grammar,
#    parser="lalr",
#    lexer="basic",
#    propagate_positions=False,
#    maybe_placeholders=False,
#    transformer=TreeToJson(),
# )
parse = json_parser.parse


def loads(jsob_str: str) -> dict:
    """parses string of JSLOB compliant content"""
    if jsob_str in ["{}", ";", "{;", "{;}", "{", ""]:
        return {}
    return parse(jsob_str)


def dumps(data: dict, tuples=False) -> str:
    """writes out dictionary as string of compliant json, to dump tuples use tuples=True, otherwise they are converted to lists to comply with standard python ``json.loads()``"""

    def spew(numz):
        out = ""
        for j in numz:
            out = out + dumps(j, tuples=tuples) + ", "
        return out.rstrip(", ")

    if type(data) is str:
        return f'"{data}"'
    elif type(data) is float or type(data) is int:
        return str(data)
    elif type(data) is list:
        return "[" + spew(data) + "]"
    elif type(data) is tuple:
        if tuples:
            return "(" + spew(data) + ")"
        else:
            return "[" + spew(data) + "]"
    elif type(data) is dict:
        out = "{"
        for k in data.keys():
            out = out + f'"{k}": ' + dumps(data[k], tuples=tuples) + ", "
        out = out.rstrip(", ") + "}"
        return out


def correct(jsob_str: str) -> str:
    """reads JSLOB compliant syntax and outputs JSON compliant syntax"""
    return dumps(parse(jsob_str))


def randsp(mins=0, maxs=3):
    return mins * " " + int(random() * maxs) * " " + ""


def randl():
    if random() > 0.5:
        return "="
    else:
        return ":"


def randc(col=False):
    if random() > 0.5:
        return ","
    elif col:
        return ";"
    else:
        return " "


def randend():
    if random() > 0.5:
        return "}"
    else:
        return ";"


def randq(s, p=0.5):
    if random() > p or not s.isalnum():
        return '"' + s + '"'
    else:
        return s


def dumpslob(data) -> str:
    """Dump Stochastically Lacerated Objects. dumps stoachastically perterbed JSLOB compliant text from tuple, list, dict"""

    def spew(numz):
        out = ""
        for j in numz:
            out = out + randsp(0, 2) + dumpslob(j) + randsp(0, 2) + ","
        return out.rstrip(",") + randc()

    if type(data) is str:
        if not data[0].isdigit() and data.replace("_", "").isalnum():
            return randq(data)
        else:
            return '"' + data + '"'
    elif type(data) is float:
        if random() > 0.3:
            return str(data) + "0" * int(random() * 2)
        else:
            return str(data)
    elif type(data) is int:
        return str(data)
    elif type(data) is list:
        return "[" + spew(data) + "]"
    elif type(data) is tuple:
        return "(" + spew(data) + ")"
    elif type(data) is dict:
        if random() > 0.5:
            out = ""
        else:
            out = "{"
        for k in data.keys():
            out = (
                out
                + randsp()
                + randq(k)
                + randl()
                + randsp()
                + str(dumpslob(data[k]))
                + randc(col=True)
            )
        out = out[0:-1] + randend()
        return out


__all__ = (
    "loads",
    "dumps",
    "correct",
    "dumpslob",
)



"""
JSONofabitch
============

sloppy json parser. messy json parser. error correcting json parser. standarization of low standards

"""

__version__: str = "0.3.0"


from .jsonofabitch import __all__
from .jsonofabitch import *



import re

# from setuptools import find_packages
import os
from setuptools import setup
from setuptools.command.build_py import build_py

# User-friendly description from README.md
current_directory = os.path.dirname(os.path.abspath(__file__))
try:
    with open(os.path.join(current_directory, "README.md"), encoding="utf-8") as f:
        long_description = f.read()
except Exception:
    long_description = ""
(__version__,) = re.findall(
    '__version__: str = "(.*)"', open("jsonofabitch/__init__.py").read()
)


class jsob_build(build_py):
    def run(self):
        from lark import Lark
        from lark.tools import standalone

        g = open(os.path.join(current_directory, "jsonofabitch/jsonofabitch.lark"), "r")
        o = open(os.path.join(current_directory, "jsonofabitch/jsob.py"), "w")
        L = Lark(g.read(), parser="lalr")
        standalone.gen_standalone(L, out=o)
        o.close()
        g.close()
        build_py.run(self)


setup(
    # Name of the package
    name="jsonofabitch",
    # Packages to include into the distribution
    version=__version__,
    license="MIT",
    # Short description of your library
    description="JSOB: Fault tolerant JSON parser. A standardization of low standards. Be a JSLOB.",
    # Long description of your library
    long_description=long_description,
    long_description_content_type="text/markdown",
    # Your name
    author="cmark0v",
    # Your email
    author_email="cmark0v@protonmail.com",
    # Either the link to your github or to your website
    url="https://github.com/cmark0v/jsonofabitch/",
    # Link from which the project can be downloaded
    download_url="",
    # List of keywords
    keywords=[
        "json",
        "parser",
        "linter",
        "autocorrect",
        "formatter",
        "jslob",
        "user input",
    ],
    # List of packages to install with this one
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: File Formats :: JSON",
        "Topic :: Software Development :: Pre-processors",
        "Topic :: File Formats",
    ],
    install_requires=[],
    requires=[],
    packages=[
        "jsonofabitch",
    ],
    cmdclass={"build_py": jsob_build},
    package_data={
        "": [os.path.join(current_directory, "jsonofabitch/jsonofabitch.lark")]
    },
    include_package_data=True,
)
